@@ -22,17 +22,18 @@
 # Creation Date : 2019-02-20 - 22:21:14
 """
 -----------
-DOCSTRING
 
-@author: Cyril Desjouy
+fdgrid init file
+
+-----------
 """
 
-__version__ = "0.7.0"
+__version__ = "0.8.8"
 
 __all__ = ['mesh',
            'domains',
            'templates',
            'graphics']
 
-from .domains import Domain, Subdomain
+from .domains import Domain, Subdomain, Obstacle
 from .mesh import Mesh, AdaptativeMesh, CurvilinearMesh
