@@ -22,21 +22,21 @@
 # Creation Date : 2019-02-13 - 07:54:54
 #
 # pylint: disable=too-many-public-methods
+# pylint: disable=too-few-public-methods
 # pylint: disable=attribute-defined-outside-init
 """
 -----------
 
-Module `domains` provides two classes:
+Module `domains` provides three main objects:
 
     * Subdomain : dataclass for subdivisions of the computational domain
+    * Obstacle : Subdomain subclass with custom boundary possibilities
     * Domain : Container for Subdomain objects
 
-It also provides the fonction `plot_subdomain`.
-
-@author: Cyril Desjouy
+-----------
 """
 
-__all__ = ['Domain', 'Subdomain']
+__all__ = ['Domain', 'Subdomain', 'Obstacle']
 
 import copy as _copy
 import itertools as _itertools
@@ -164,11 +164,12 @@ class Domain:
         """
         Append 'val' to the Domain object.
 
-        Arguments :
+        Parameters
         ----------
         val : The object to add to the instance.
               Must be a Subdomain object or a list/tuple of Subdomain objects
         key : new key for val
+
         """
 
         if isinstance(val, Subdomain):
@@ -213,7 +214,8 @@ class Domain:
     def split(self, key, index, nbc=None, axis=None):
         """ Split 'key' subdomain.
 
-        Arguments:
+        Parameters
+        ----------
             index : int|tuple|list
             nbc : Only if index is a sequence. New bc for the domain defined by 'index'
             axis : 0 or 1. Along which axis to split
@@ -400,7 +402,7 @@ class Domain:
             r = getattr(sub, 'rx')
             axis = 1
 
-        nbc = sub.bc.replace('P', 'R')
+        nbc = sub.bc.replace('P', 'W')
 
         for bound in bounds:
             if set(range(bound[0], bound[1]+1)).issuperset(set(r)):
@@ -417,13 +419,13 @@ class Domain:
         self.additional_rigid_bc = []
 
         for sub in self:
-            if sub.xz[axis] == 0 and sub.bc[axis] == 'R':
+            if sub.xz[axis] == 0 and sub.bc[axis] == 'W':
                 if axis == 0:
                     self.additional_rigid_bc.append((0, sub.sz))
                 elif axis == 1:
                     self.additional_rigid_bc.append((sub.sx, 0))
 
-            elif sub.xz[axis+2] == N-1 and sub.bc[axis+2] == 'R':
+            elif sub.xz[axis+2] == N-1 and sub.bc[axis+2] == 'W':
                 if axis == 0:
                     self.additional_rigid_bc.append((-1, sub.sz))
                 elif axis == 1:
@@ -551,16 +553,21 @@ class Domain:
 
 @_dataclasses.dataclass
 class Subdomain:
-    """ Subdomain of the computation domain.
+    """ Subdivision of the computation domain.
 
     Parameters
     ----------
 
-    xz : Coordinates of the Subdomain : left, bottom, right, top
-    bc : Boundary conditions. Must be a string of 4 chacracter among 'A', 'R', 'Z' and 'P'
-    key : Key of the subdomain. Optional
-    axis : 0 or 1. The direction of the subdomain if relevant. Optional.
-    tag : str. The type of Subdomain. Optional
+    xz : list
+        Coordinates of the Subdomain : left, bottom, right, top
+    bc : str
+        Boundary conditions. Must be a string of 4 characters among 'A|W|Z|P|R'
+    key : int, optional
+        Key of the subdomain.
+    axis : {0, 1}, optional
+        The direction of the subdomain if relevant.
+    tag : str, optional
+        The type of Subdomain.
     """
 
     xz: tuple
@@ -674,15 +681,21 @@ class Subdomain:
         return self.bc
 
     def split(self, index, nbc=None, axis=None):
-        """ Split the subdomain along axis'.
+        """
+        Split the subdomain into several new subdomains.
+
         If index is integer, split into two Subdomain objects at 'index'.
         If index is tuple/list (len(index) must be 2), split into 3 (or less)
         Subdomain objects.
 
-        Arguments:
-            index : int|tuple|list
-            nbc : Only if index is a sequence. New bc for the domain defined by 'index'
-            axis : 0 or 1. Along which axis to split
+        Parameters
+        ----------
+            index : {int, tuple, list}
+                Where to split the subdomain
+            nbc : str
+                Only if index is a sequence. New bc for the domain defined by 'index'
+            axis : {0, 1}
+                Along which axis to split
         """
 
         subkey = (i for i in range(1, 4))
@@ -766,13 +779,13 @@ class Subdomain:
 
         elif axis != self.axis and self.axis == 0:
             bc = ['X.X.']*len(xz)
-            bc[0] = '{}.X.'.format(self.bc[0])
-            bc[-1] = 'X.{}.'.format(self.bc[2])
+            bc[0] = f'{self.bc[0]}.X.'
+            bc[-1] = f'X.{self.bc[2]}.'
 
         elif axis != self.axis and self.axis == 1:
             bc = ['.X.X']*len(xz)
-            bc[0] = '.{}.X'.format(self.bc[1])
-            bc[-1] = '.X.{}'.format(self.bc[3])
+            bc[0] = f'.{self.bc[1]}.X'
+            bc[-1] = f'.X.{self.bc[3]}'
 
         else:
             bc = [self.bc]*len(xz)
@@ -789,3 +802,248 @@ class Subdomain:
                 tags[i] = 'W'
 
         return tags
+
+
+@_dataclasses.dataclass
+class Edges:
+    """ Edges description for moving boundaries. """
+
+    func: str = None
+    wav: str = None
+    f0_t: float = 0
+    f0_n: float = 0
+    phi_t: float = 0
+    phi_n: float = 0
+    pn: _np.ndarray = 0    # profile of the wall
+    pt: _np.ndarray = 0    # profile of the wall
+    vn: _np.ndarray = 0    # time evolution of the wall
+    vt: _np.ndarray = 0    # time evolution of the wall
+    sx: slice = 0
+    sz: slice = 0
+    axis: int = 0
+
+
+class Obstacle(Subdomain):
+    """ Obstacle object.
+
+    Parameters
+    ----------
+
+    xz : Coordinates of the Subdomain : left, bottom, right, top
+    bc : Boundary conditions. Must be a string of 4 characters among 'W|Z|V'
+    key : Key of the subdomain. Optional
+    axis : 0 or 1. The direction of the subdomain if relevant. Optional.
+    tag : str. The type of Subdomain. Optional
+    """
+
+    def __post_init__(self):
+        super().__post_init__()
+        self.edges = []
+
+    def set_moving_bc(self, *args):
+        """
+        Set parameters for moving bc. For each moving bc of the obstacle,
+        you can specify the parameters as a dictionnary containing the
+        following optional keys :
+
+        Parameters
+        ----------
+        f_n : float, str
+            frequency of the normal velocity.
+        f_t : float
+            frequency of the tangential velocity.
+        A_n : float
+            Amplitude of the normal velocity
+        A_t : float
+            Amplitude of the tangential velocity
+        phi_n : float
+            Phase of the normal velocity
+        phi_t : float
+            Phase of the tangential velocity
+        profile_r : {'sine', 'tukey', 'flat'}
+            Function for normal velocity profile construction
+        profile_t : {'sine', 'tukey', 'flat'}
+            Function for tangential velocity profile construction
+        kwargs_n : dict
+            optional arguments for profile_r
+        kwargs_t : dict
+            optional arguments for profile_t
+
+        Note
+        ----
+
+        `profile_n` and `profile_r` arguments can be:
+
+        * 'sine' : sinus profile => n : wavelength (default: 2 for half period)
+        * 'tukey' : tapered cosine profile => alpha : edge width (default: 0.2)
+        * 'flat' : flat profile
+
+        Examples
+        --------
+
+        >>> obs1 = Obstacle([10, 10, 100, 100], 'VVWV')
+
+        >>> obs1.set_moving_bc({'f': 100, 'A': 2.1, 'profile': 'sine',
+                                'f_t': 1000, 'A_t': 0.1, 'profile_t': 'flat'},
+                               {'f': 500, 'A': -1.5, 'profile': 'tukey'},
+                               {'f': 500, 'A': 1.5 'profile': 'tukey'})
+
+        """
+
+        args = iter(list(args) + (4-len(args))*[{}])
+        self.setup = [{} if b == 'W' else next(args) for b in self.bc]
+
+    def _parse_bc_setup_n(self, setup):
+
+        prof = setup.get('profile_n', setup.get('profile', 'flat'))
+        prof = getattr(self, prof, self.flat)
+        wav = setup.get('wav', None)
+        func = setup.get('func', None)
+
+        f = setup.get('f_n', setup.get('f', 0))
+        phi = setup.get('phi_n', setup.get('phi', 0))
+        A = setup.get('A_n', setup.get('A', 0))
+        kwargs = setup.get('kwargs_n', setup.get('kwargs', {}))
+
+        return prof, f, phi, A, kwargs, wav, func
+
+    def _parse_bc_setup_t(self, setup):
+
+        prof = setup.get('profile_t', 'flat')
+        prof = getattr(self, prof, self.flat)
+
+        f = setup.get('f_t', 0)
+        phi = setup.get('phi_t', 0)
+        A = setup.get('A_t', 0)
+        kwargs = setup.get('kwargs_t', {})
+
+        return prof, f, phi, A, kwargs
+
+    def make_moving_bc(self, x, z, LOG=False):
+        """ Construct moving boundaries. """
+
+        if hasattr(self, 'setup'):
+
+            for i in range(4):
+
+                if len(x.shape) == 2:
+                    self._make_moving_bc_curvilinear(i, x, z)
+
+                else:
+                    self._make_moving_bc_cartesian(i, x, z)
+
+            if LOG:
+                for e in self.edges:
+                    _, ax = _plt.subplots(1, 2)
+                    ax[0].plot(e.pn)
+                    ax[0].grid()
+                    ax[0].set_title('Numerical domain')
+
+                    if e.axis == 0:
+                        ax[1].plot(z[e.sx, e.sz], e.pn)
+                    if e.axis == 1:
+                        ax[1].plot(x[e.sx, e.sz], e.pn)
+
+                    ax[1].set_title('Physical domain')
+                    ax[1].grid()
+                    _plt.tight_layout()
+
+    def _make_moving_bc_cartesian(self, i, x, z):
+
+        prof_n, f_n, phi_n, A_n, kw_n, wav, func = self._parse_bc_setup_n(self.setup[i])
+        prof_t, f_t, phi_t, A_t, kw_t = self._parse_bc_setup_t(self.setup[i])
+
+        d = {'axis': i % 2, 'wav': wav, 'func': func}
+
+        if i % 2 == 0:    # following x
+            d['sx'] = self.xz[i]
+            d['sz'] = self.sz
+            if prof_n:
+                d.update({'f0_n': f_n, 'phi_n': phi_n})
+                d['pn'] = A_n*prof_n(z, self.sz, **kw_n)
+            if prof_t:
+                d.update({'f0_t': f_t, 'phi_t': phi_t})
+                d['pt'] = A_t*prof_t(z, self.sz, **kw_t)
+
+        else:           # following z
+            d['sx'] = self.sx
+            d['sz'] = self.xz[i]
+            if prof_n:
+                d.update({'f0_n': f_n, 'phi_n': phi_n})
+                d['pn'] = A_n*prof_n(x, self.sx, **kw_n)
+            if prof_t:
+                d.update({'f0_t': f_t, 'phi_t': phi_t})
+                d['pt'] = A_t*prof_t(x, self.sx, **kw_t)
+
+        if prof_n or prof_t:
+            self.edges.append(Edges(**d))
+
+    def _make_moving_bc_curvilinear(self, i, x, z):
+
+        prof_n, f_n, phi_n, A_n, kw_n, wav, func = self._parse_bc_setup_n(self.setup[i])
+        prof_t, f_t, phi_t, A_t, kw_t = self._parse_bc_setup_t(self.setup[i])
+
+        d = {'axis': i % 2, 'wav': wav, 'func': func}
+
+        if i % 2 == 0:    # following x
+            d['sx'] = self.xz[i]
+            d['sz'] = self.sz
+            if prof_n:
+                d.update({'f0_n': f_n, 'phi_n': phi_n})
+                d['pn'] = A_n*prof_n(z[self.xz[i], :], self.sz, **kw_n)
+            if prof_t:
+                d.update({'f0_t': f_t, 'phi_t': phi_t})
+                d['pt'] = A_t*prof_t(z[self.xz[i], :], self.sz, **kw_t)
+
+        else:           # following z
+            d['sx'] = self.sx
+            d['sz'] = self.xz[i]
+            if prof_n:
+                d.update({'f0_n': f_n, 'phi_n': phi_n})
+                d['pn'] = A_n*prof_n(x[:, self.xz[i]], self.sx, **kw_n)
+            if prof_t:
+                d.update({'f0_t': f_t, 'phi_t': phi_t})
+                d['pt'] = A_t*prof_t(x[:, self.xz[i]], self.sx, **kw_t)
+
+        if prof_n or prof_t:
+            self.edges.append(Edges(**d))
+
+    @staticmethod
+    def sine(c, sc, **kwargs):
+        """ Sine profile. """
+
+        L = c[sc.stop-1] - c[sc.start]
+        n = kwargs.get('n', 2)
+        kL = 2*_np.pi/(n*L)
+
+        return _np.sin(kL*(c[sc] - c[sc.start]))
+
+    @staticmethod
+    def tukey(c, sc, **kwargs):
+        """ Tapered cosine profile. """
+
+        L = c[sc.stop-1] - c[sc.start]
+        N = sc.stop - sc.start
+
+        alpha = kwargs.get('alpha', 0.2)
+        edges = L*alpha/2
+
+        Nmin = max(1, abs(c - (c[sc.start] + edges)).argmin() - sc.start)
+        Nmax = min(N-1, abs(c - (c[sc.stop] - edges)).argmin() - sc.start)
+
+        nmin = _np.linspace(0, alpha*(N-1)/2, Nmin)
+        nmax = _np.linspace((N+1)*(1-alpha/2), N, N - Nmax)
+
+        tukmin = 0.5*(1 + _np.cos(_np.pi*(2*nmin/(alpha*N) + 1)))
+        tukmax = 0.5*(1 + _np.cos(_np.pi*(2*nmax/(alpha*N) - 2/alpha + 1)))
+
+        tukmin[0] = 0
+        tukmax[-1] = 0
+
+        return _np.concatenate((tukmin, _np.ones(max(0, Nmax-Nmin)), tukmax))
+
+    @staticmethod
+    def flat(*args):
+        """ flat profile. """
+
+        return _np.ones(args[1].stop-args[1].start)
