@@ -23,13 +23,16 @@
 #
 # pylint: disable=too-many-instance-attributes
 # pylint: disable=too-many-arguments
-# pylint: disable=too-many-locals
 """
 -----------
 
-Submodule `mesh` provides the mesh classes.
+Module `mesh` provides three classes to build meshes:
 
-@author: Cyril Desjouy
+    * :py:class:`Mesh`: Build regular cartesian mesh
+    * :py:class:`AdaptativeMesh`: Build adaptative cartesian mesh
+    * :py:class:`CurvilinearMesh`: Build curvilinear mesh
+
+-----------
 """
 
 __all__ = ['Mesh', 'AdaptativeMesh', 'CurvilinearMesh']
@@ -48,22 +51,35 @@ from fdgrid.templates import curv as _curv
 
 
 class Mesh:
-    """ Mesh Class : Construct grid with constant spatial steps
+    """ Build cartesian regular grid
 
     Parameters
     ----------
+    shape : tuple
+        Size of the domain. Must be a tuple with two int objects.
+    step : tuple
+        Spatial steps. Must be a tuple with two float objects.
+    origin : tuple, optional
+        Origin of the grid. Must be a tuple with two int objects.
+    bc : {'[ARZPW][ARZPW][ARZPW][ARZPW]'}, optional
+        Boundary conditions. Must be a 4 characters string.
+    obstacles : :py:class:`fdgrid.domains.Domain`, optional
+        Obstacles in the computation domain.
+    Npml : int, optional
+        Number of points of the absorbing area (only if 'A' in `bc`).
+    stencil : int, optional
+        Size of the finite difference stencil (used by :py:mod:`nsfds2`).
+
+    See also
+    --------
+    :py:class:`AdaptativeMesh`,
+    :py:class:`CurvilinearMesh`,
+    :py:mod:`fdgrid.templates`
 
-    shape : Size of the domain. Must be a 2 elements tuple of int
-    step : Spatial steps. Must be a 2 elements tuple of float
-    origin : Origin of the grid. Must be a 2 elements tuple of int. Optional
-    bc : Boundary conditions. Must be a string of 4 chacracter among 'A', 'R', 'Z' and 'P'
-    obstacles : Domain object. Set of obstacles to consider
-    Npml : Int. Number of points of the absorbing area (if 'A' is in bc). Optional
-    stencil : Int. Size of the finite difference stencil that will be used by nsfds2
     """
 
     def __init__(self, shape, step, origin=(0, 0),
-                 bc='RRRR', obstacles=None, Npml=15, stencil=11):
+                 bc='WWWW', obstacles=None, Npml=15, stencil=11):
 
 
         self.shape = shape
@@ -82,6 +98,7 @@ class Mesh:
 
         self._make_grid()
         self._find_subdomains()
+        self._make_moving_bc()
 
     def _make_grid(self):
         """ Make grid. """
@@ -90,11 +107,11 @@ class Mesh:
         self.z = (_np.arange(self.nz)-self.iz0)*self.dz
 
     def N(self, axis):
-        """ Return number of point following 'axis'. """
+        """ Returns the number of points following `axis`. """
         return self.nx if axis == 0 else self.nz if axis == 1 else None
 
     def du(self, axis):
-        """ Return base spacial step following 'axis'. """
+        """ Returns the spacial base step following `axis`. """
         return self.dx if axis == 0 else self.dz if axis == 1 else None
 
     def _limits(self):
@@ -103,7 +120,16 @@ class Mesh:
         return xlim, zlim
 
     def _find_subdomains(self):
-        """ Divide the computation domain in subdomains. """
+        """ Divide the computation domain into subdomains.
+
+        dx/dzdomains : for x/z derivatives
+        fx/fzdomains : for x/z filters
+
+        dsdomains : smallest domain among dx/dzdomains
+        fsdomains : smallest domain among fx/fzdomains
+        adomains : for PML
+
+        """
 
         self.domain = ComputationDomains((self.nx, self.nz), self.obstacles,
                                          self.bc, self.stencil, self.Npml)
@@ -125,8 +151,8 @@ class Mesh:
 
         regex = [r'[^P].P.', r'P.[^P].', r'.[^P].P', r'.P.[^P]']
 
-        if not _re.match(r'^[ZRAP]*$', self.bc):
-            raise _exceptions.BoundaryConditionError("bc must be combination of 'ZRAP'!")
+        if not _re.match(r'^[ZRAPW]*$', self.bc):
+            raise _exceptions.BoundaryConditionError("bc must be combination of 'ZRAPW'!")
 
         if any(_re.match(r, self.bc) for r in regex):
             msg = "periodic condition must be on both sides of the domain,"
@@ -136,20 +162,39 @@ class Mesh:
     def _check_grid(self):
 
         if self.Npml < self.stencil:
-            raise _exceptions.GridError("Number of points of PML must be greater than stencil")
+            raise _exceptions.GridError("Number of points of PML must be larger than stencil")
 
         if self.ix0 > self.nx or self.iz0 > self.nz:
             raise _exceptions.GridError("Origin of the domain must be in the domain")
 
-    def plot_domains(self, legend=False, N=4, figsize=(9, 18), filename=None):
+    def _make_moving_bc(self):
+
+        for obs in self.obstacles:
+            obs.make_moving_bc(self.x, self.z)
+
+    def plot_domains(self, legend=False, N=4, figsize=(9, 18),
+                     bc_profiles=False, filename=None):
         """ Plot a scheme of the computation domain higlighting the subdomains.
 
-            Parameters
-            ----------
-            legend: Show mesh legend. bool
-            N: Keep one point over N. int
-            filename: save to filename. string
-            figsize: size of the figure. tuple
+        Parameters
+        ----------
+        legend: bool, optional
+            Show legend.
+        N: int, optional
+            Plot the grid with one point over N.
+        filename: string, optional
+            Save to `filename`.
+        figsize: tuple, optional
+            Size of the figure.
+        bc_profiles: bool, optional
+            Show velocity profiles for moving boundaries
+
+        See also
+        --------
+        :py:meth:`plot_grid`,
+        :py:meth:`plot_physical`,
+        :py:meth:`plot_xz`
+
         """
 
         _, axes = _plt.subplots(2, 1, figsize=figsize)
@@ -184,34 +229,53 @@ class Mesh:
                                   bg='k', fg='w')
             print(self.domain)
 
+        if bc_profiles:
+            for ax in axes:
+                _graphics.plot_bc_profiles(ax, self.x, self.z, self.obstacles)
+
+
         _plt.tight_layout()
 
         if isinstance(filename, str):
             _plt.savefig(filename)
 
-    def plot_grid(self, figsize=(9, 5), N=4, axis=False, pml=False, filename=None):
-        """ Grid representation.
+    def plot_grid(self, figsize=(9, 5), N=4, axis=False, legend=False,
+                  pml=False, bc_profiles=False, probes=False, filename=None):
+        """ Numerical grid representation.
 
-            Parameters
-            ----------
-            legend: Show mesh legend. bool
-            N: Keep one point over N. int
-            filename: save to filename. string
-            figsize: size of the figure. tuple
-            pml: Show PML. bool
+        Parameters
+        ----------
+
+        legend: bool, optional
+            Show legend.
+        N: int, optional
+            Plot the grid with one point over N.
+        filename: string, optional
+            Save to `filename`.
+        figsize: tuple, optional
+            Size of the figure.
+        pml: bool, optional
+            Show absorbing area in the figure.
+        bc_profiles: bool, optional
+            Show velocity profiles for moving boundaries
+
+        Important
+        ---------
+        The curves :math:`dx'/dx` are not representative on the edges because
+        of the gradient.
+
+        See also
+        --------
+        :py:meth:`plot_physical`,
+        :py:meth:`plot_xz`
 
-            BUG
-            ---
-            Note that the curves dx'/dx are not representative on the edges because
-            of the gradient.
         """
 
-        nullfmt = _ticker.NullFormatter()         # no labels
-
         fig, ax_c = _plt.subplots(figsize=figsize)
         fig.subplots_adjust(.1, .1, .95, .95)
 
-        _graphics.plot_subdomains(ax_c, self.x, self.z, self.obstacles, facecolor='k')
+        _graphics.plot_subdomains(ax_c, self.x, self.z, self.obstacles,
+                                  legend=legend, facecolor='k')
         _graphics.plot_grid(ax_c, self.x, self.z, N)
 
         ax_c.set_xlim(self.x.min(), self.x.max())
@@ -220,9 +284,15 @@ class Mesh:
         ax_c.set_ylabel(r'$z$ [m]')
         ax_c.set_aspect('equal')
 
+        if probes:
+            _ = [ax_c.plot(self.x[i], self.z[j], 'ro') for i, j in probes]
+
         if pml:
             _graphics.plot_pml(ax_c, self.x, self.z, self.bc, self.Npml)
 
+        if bc_profiles:
+            _graphics.plot_bc_profiles(ax_c, self.x, self.z, self.obstacles)
+
         if axis:
             divider = _axes_grid1.make_axes_locatable(ax_c)
             ax_xa = divider.append_axes('top', size='30%', pad=0.1)
@@ -242,14 +312,14 @@ class Mesh:
 
             for ax in [ax_xa, ax_xb]:
                 ax.set_xlim(ax_c.get_xlim())
-                ax.xaxis.set_major_formatter(nullfmt)  # no label
+                ax.xaxis.set_major_formatter(_ticker.NullFormatter())  # no label
                 for j in self._limits()[0]:
                     if j[-1] == 'o':
                         ax.axvspan(self.x[j[0]], self.x[j[1]], facecolor='k', alpha=0.5)
 
             for ax in [ax_za, ax_zb]:
                 ax.set_ylim(ax_c.get_ylim())
-                ax.yaxis.set_major_formatter(nullfmt)  # no label
+                ax.yaxis.set_major_formatter(_ticker.NullFormatter())  # no label
                 for j in self._limits()[1]:
                     if j[-1] == 'o':
                         ax.axhspan(self.z[j[0]], self.z[j[1]], facecolor='k', alpha=0.5)
@@ -257,13 +327,36 @@ class Mesh:
         if isinstance(filename, str):
             _plt.savefig(filename)
 
+    def plot_physical(self, figsize=(9, 4), legend=False,
+                      pml=False, bc_profiles=False, probes=False, filename=None):
+        """ Physical grid representation.
+
+
+        See also
+        --------
+        :py:meth:`plot_grid`,
+        :py:meth:`plot_xz`
+
+        """
+
+        self.plot_grid(figsize=figsize, pml=pml, legend=legend,
+                       bc_profiles=bc_profiles, probes=probes, filename=filename)
+
     def plot_xz(self, figsize=(9, 4), filename=None):
         """ Plot x & z axis.
 
-            Parameters
-            ----------
-            filename: save to filename. string
-            figsize: size of the figure. tuple
+        Parameters
+        ----------
+        filename : str, optional
+            Save figure to filename.
+        figsize : tuple, optional
+            Size of the figure.
+
+        See also
+        --------
+        :py:meth:`plot_physical`,
+        :py:meth:`plot_xz`
+
         """
 
         _, axes = _plt.subplots(2, 2, figsize=figsize)
@@ -308,8 +401,9 @@ class Mesh:
         s = 'Cartesian {}x{} points grid with {} boundary conditions:\n\n'
         s += '\t* Spatial step : {}\n'.format((self.dx, self.dz))
         s += '\t* Origin       : {}\n'.format((self.ix0, self.iz0))
-        s += '\t* Points in PML: {}\n'.format(self.Npml)
-        s += '\t* Max stencil   : {}\n'.format(self.stencil)
+        if 'A' in self.bc:
+            s += '\t* Points in PML: {}\n'.format(self.Npml)
+        s += '\t* Max stencil  : {}\n'.format(self.stencil)
 
         return s.format(self.nx, self.nz, self.bc)
 
@@ -318,25 +412,26 @@ class Mesh:
 
 
 class AdaptativeMesh(Mesh):
-    """ AdaptativeMesh Class : Construct grid with adaptative spatial steps
+    """ Build cartesian adaptative grid
 
     Parameters
     ----------
+    dilatation : float
+        dilatation rate over `Nd` points.
+    Nd : int
+        Number of point of adaptative areas.
+    only_pml : bool
+        Only adapt grid in PML areas.
+
+    See also
+    --------
+    :py:class:`Mesh`,
+    :py:mod:`fdgrid.templates`
 
-    shape : Size of the domain. Must be a 2 elements tuple of int
-    step : Spatial steps. Must be a 2 elements tuple of float
-    origin : Origin of the grid. Must be a 2 elements tuple of int. Optional
-    bc : Boundary conditions. Must be a string of 4 chacracter among 'A', 'R', 'Z' and 'P'
-    obstacles : Domain object. Set of obstacles to consider
-    Npml : Int. Number of points of the absorbing area (if 'A' is in bc). Optional
-    stencil : Int. Size of the finite difference stencil that will be used by nsfds2
-    dilatation : dilatation rate over Nd points. Float.
-    Nd : Number of point of adaptative areas. Integer
-    only_pml : Adaptative mesh only in PML areas. Boolean.
     """
 
     def __init__(self, shape, step, origin=(0, 0),
-                 bc='RRRR', obstacles=None, Npml=15, stencil=11,
+                 bc='WWWW', obstacles=None, Npml=15, stencil=11,
                  dilatation=3., Nd=23, only_pml=False):
 
         self.only_pml = only_pml
@@ -382,14 +477,14 @@ class AdaptativeMesh(Mesh):
         if self.bc[axis] == 'A':
             a[:self.Npml] = self.stretch_PML
 
-        elif self.bc[axis] in ['R', 'Z']:
+        elif self.bc[axis] in ['W', 'Z']:
             a[start:start+self.stencil] = self.amin
             a[start+self.stencil:start+self.stencil+self.N] = self.stretch[::-1]
 
         if self.bc[axis+2] == 'A':
             a[stop-self.Npml+1:] = self.stretch_PML[::-1]
 
-        elif self.bc[axis+2] in ['R', 'Z']:
+        elif self.bc[axis+2] in ['W', 'Z']:
             a[stop-self.stencil:stop+1] = self.amin
             a[stop-self.stencil-self.N+1:stop-self.stencil+1] = self.stretch
 
@@ -545,7 +640,8 @@ class AdaptativeMesh(Mesh):
         s = 'Adaptative cartesian {}x{} points grid with {} boundary conditions:\n\n'
         s += '\t* Spatial step  : {}\n'.format((self.dx, self.dz))
         s += '\t* Origin        : {}\n'.format((self.ix0, self.iz0))
-        s += '\t* Points in PML : {}\n'.format(self.Npml)
+        if 'A' in self.bc:
+            s += '\t* Points in PML : {}\n'.format(self.Npml)
         s += '\t* Max stencil   : {}\n'.format(self.stencil)
         s += '\t* Dilatation over {} pts  : {:.1f} %\n'.format(self.N, (1-self.amin)*100)
 
@@ -553,24 +649,22 @@ class AdaptativeMesh(Mesh):
 
 
 class CurvilinearMesh(Mesh):
-    """ CurvilinearMesh Class : Construct grid using curvilinear coordinates
+    """ Build curvilinear grid
 
     Parameters
     ----------
+    fcurvxz : function, optional
+        Function taking as input arguments the numerical coordinates ('xn', 'yn')
+        and returning the physical coordinates ('xp', 'zp').
 
-    shape : Size of the domain. Must be a 2 elements tuple of int
-    step : Spatial steps. Must be a 2 elements tuple of float
-    origin : Origin of the grid. Must be a 2 elements tuple of int. Optional
-    bc : Boundary conditions. Must be a string of 4 chacracter among 'A', 'R', 'Z' and 'P'
-    obstacles : Domain object. Set of obstacles to consider
-    Npml : Int. Number of points of the absorbing area (if 'A' is in bc). Optional
-    stencil : Int. Size of the finite difference stencil that will be used by nsfds2
-    fcurvxz : function taking as input arguments the numerical coordinates (xn/yn)
-              and returning the physical coordinates (xp/zp)
+    See also
+    --------
+    :py:class:`Mesh`,
+    :py:mod:`fdgrid.templates`
     """
 
     def __init__(self, shape, step, origin=(0, 0),
-                 bc='RRRR', obstacles=None, Npml=15, stencil=11, fcurvxz=_curv):
+                 bc='WWWW', obstacles=None, Npml=15, stencil=11, fcurvxz=_curv):
 
         if not fcurvxz:
             self.fcurvxz = _curv
@@ -579,15 +673,24 @@ class CurvilinearMesh(Mesh):
 
         super().__init__(shape, step, origin, bc, obstacles, Npml, stencil)
 
-    def plot_physical(self, figsize=(9, 4), legend=False, pml=False, filename=None):
+    def plot_physical(self, figsize=(9, 4), legend=False,
+                      pml=False, bc_profiles=False, probes=False, filename=None):
         """ Plot physical and numerical domains.
 
-            Parameters
-            ----------
-            legend: Show mesh legend. bool
-            filename: save to filename. string
-            figsize: size of the figure. tuple
-            pml: Show PML. bool
+        Parameters
+        ----------
+        legend: bool, optional
+            Show legend.
+        N: int, optional
+            Plot the grid with one point over N.
+        filename: string, optional
+            Save to `filename`.
+        figsize: tuple, optional
+            Size of the figure.
+        pml: bool, optional
+            Show absorbing area in the figure.
+        bc_profiles: bool, optional
+            Show velocity profiles for moving boundaries
         """
 
         edgecolor = 'k'
@@ -607,16 +710,25 @@ class CurvilinearMesh(Mesh):
                                   legend=legend,
                                   edgecolor=edgecolor, facecolor=facecolor, alpha=alpha)
 
+        _plt.tight_layout()
+
+        if probes:
+            _ = [axes[0].plot(self.x[i], self.z[j], 'ro') for i, j in probes]
+            _ = [axes[1].plot(self.xp[i, j], self.zp[i, j], 'ro') for i, j in probes]
+
         if pml:
             _graphics.plot_pml(axes[0], self.x, self.z, self.bc, self.Npml)
             _graphics.plot_pml(axes[1], self.xp, self.zp, self.bc, self.Npml)
 
+        if bc_profiles:
+            _graphics.plot_bc_profiles(axes[0], self.x, self.z, self.obstacles)
+            _graphics.plot_bc_profiles(axes[1], self.xp, self.zp, self.obstacles)
+
         for ax in axes:
             ax.set_aspect('equal')
             ax.set_xlabel(r'x [m]')
             ax.set_ylabel(r'z [m]')
 
-        _plt.tight_layout()
 
         if isinstance(filename, str):
             _plt.savefig(filename)
@@ -657,13 +769,19 @@ class CurvilinearMesh(Mesh):
         if _np.abs(gcl_x).max() > 1e-8 or _np.abs(gcl_z).max() > 1e-8:
             print('GCL (x) : ', _np.abs(gcl_x).max())
             print('GCL (z) : ', _np.abs(gcl_z).max())
-            raise ValueError('Geometric Conservation Laws not verified')
+            raise _exceptions.GridError('Geometric Conservation Laws not verified')
+
+    def _make_moving_bc(self):
+
+        for obs in self.obstacles:
+            obs.make_moving_bc(self.x, self.z)
 
     def __str__(self):
         s = 'Curvilinear {}x{} points grid with {} boundary conditions:\n\n'
         s += '\t* Spatial step  : {}\n'.format((self.dx, self.dz))
         s += '\t* Origin        : {}\n'.format((self.ix0, self.iz0))
-        s += '\t* Points in PML : {}\n'.format(self.Npml)
+        if 'A' in self.bc:
+            s += '\t* Points in PML : {}\n'.format(self.Npml)
         s += '\t* Max stencil   : {}\n'.format(self.stencil)
 
         return s.format(self.nx, self.nz, self.bc)
@@ -679,5 +797,5 @@ if __name__ == "__main__":
     obstcle = templates.plus(*shp)
 
 
-    mesh1 = Mesh(shp, stps, orgn, obstacles=obstcle, bc='PAPZ')
-    mesh1.plot_domains(legend=True, N=2)
+    mesh = Mesh(shp, stps, orgn, obstacles=obstcle, bc='PAPZ')
+    mesh.plot_domains(legend=True, N=2)
